
import typer

from typing import Optional
from frenchmaid import __app_name__, __version__

from .switches import cleanSwitch as clS, ignoreSwitch as igS

app = typer.Typer(help="A handy tool to delete all junk!", no_args_is_help=True)

ignoreSwitch = igS.IgnoreSwitch()
cleanSwitch = clS.CleanSwitch()

def _version_callback(value: bool) -> None:
    if value:
        typer.echo(f"{__app_name__} v{__version__}")
        raise typer.Exit()

@app.callback()
def main(
    version: Optional[bool] = typer.Option(
        None,
        "--version",
        "-v",
        help="View the CLI app version",
        callback=_version_callback,
        is_eager=True
    )
) -> None: return

@app.command(help="Delete all instances of junk folders")
def clean(folder: Optional[str] = typer.Argument(None, help="pycache, mypy")) -> None:
    if (folder):
        cleanSwitch.indirect(folder)
    else:
        cleanSwitch.indirect("all")
    raise typer.Exit()

@app.command(help="Add junk folders to ignore files", no_args_is_help=True)
def ignore(platform: Optional[str] = typer.Argument(..., help="git, docker")) -> None:
    ignoreSwitch.indirect(platform)
    raise typer.Exit()


import os

from ..modules import clean

directory: str = os.getcwd()

class CleanSwitch():
    def indirect(self, folder: str):
        switch = getattr(self, folder, lambda: print(f'''\nThis folder type isn't supported... yet. You can request for "{folder}" support at https://github.com/lewisjr/frenchmaid/issues"'''))
        return switch()

    def all(self) -> None:
        clean.clean(directory, 'all')
        return

    def pycache(self) -> None:
        clean.clean(directory, '__pycache__')
        return

    def mypy(self) -> None:
        clean.clean(directory, '.mypy_cache')
        return




def git(directory: str) -> None:
    try:
        f = open(".gitignore", "x")
        f.write("__pycache__/*\n.mypy_cache/")
        f.close()
        print(f'\nSuccessfully created a ".gitignore" in your "{directory}" folder. It now ignores all pycache and mypy cache folders!')
        return
    except:
        f_read = open(".gitignore", "r")
        f_write = open(".gitignore", "a")
        f_data = f_read.read()
        array: list[str] = []

        if "__pycache__/*" not in f_data:
            f_write.write("\n__pycache__/*")
            array.append('pycache')

        if ".mypy_cache/" not in f_data:
            f_write.write("\n.mypy_cache/")
            array.append('mypy')

        f_read.close()
        f_write.close()

        if len(array) == 0:
            print(f"""\nLooks like the ".gitignore" file in your "{directory}" directory already ignores all pycache and mypy cache folders!""")
        elif len(array) == 1 and array[0] == "pycache":
            print(f"""\nSuccessfully added all pycache folders to the ".gitignore" file in your "{directory}" directory!""")
        elif len(array) == 1 and array[0] == "mypy":
            print(f"""\nSuccessfully added all mypy cache folders to the ".gitignore" file in your "{directory}" directory!""")
        else:
            print(f"""\nSuccessfully added all pycache and mypy cache folders to the ".gitignore" file in your "{directory}" directory!""")

        return

def docker(directory: str) -> None:
    try:
        f = open(".dockerignore", "x")
        f.write("*__pycache__*\n.mypy_cache")
        f.close()
        print(f'\nSuccessfully created a ".dockerignore" in your "{directory}" folder. It now ignores all pycache and mypy cache folders!')
        return
    except:
        f_read = open(".dockerignore", "r")
        f_write = open(".dockerignore", "a")
        f_data = f_read.read()
        array: list[str] = []

        if "*__pycache__*" not in f_data:
            f_write.write("\n*__pycache__*")
            array.append('pycache')

        if ".mypy_cache" not in f_data:
            f_write.write("\n.mypy_cache")
            array.append('mypy')

        f_read.close()
        f_write.close()

        if len(array) == 0:
            print(f"""\nLooks like the ".dockerignore" file in your "{directory}" directory already ignores all pycache and mypy cache folders!""")
        elif len(array) == 1 and array[0] == "pycache":
            print(f"""\nSuccessfully added all pycache folders to the ".dockerignore" file in your "{directory}" directory!""")
        elif len(array) == 1 and array[0] == "mypy":
            print(f"""\nSuccessfully added all mypy cache folders to the ".dockerignore" file in your "{directory}" directory!""")
        else:
            print(f"""\nSuccessfully added all pycache and mypy cache folders to the ".dockerignore" file in your "{directory}" directory!""")

        return


import os, sys

from ..modules import ignore

platf = sys.platform
directory_path: str = os.getcwd()
directory: str = ''

if platf == "win32":
    directory = directory_path[str(directory_path).rfind("\\")+1:len(directory_path)]
else:
    directory = directory_path[str(directory_path).rfind("/")+1:len(directory_path)]

class IgnoreSwitch():
    def indirect(self, platform: str):
        switch = getattr(self, platform, lambda: print(f'''\nThis platform isn't supported... yet. You can request for "{platform}" support at https://github.com/lewisjr/frenchmaid/issues"'''))
        return switch()

    def git(self) -> None:
        ignore.git(directory)
        return

    def docker(self) -> None:
        ignore.docker(directory)
        return



import setuptools

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setuptools.setup(
    name="frenchmaid",
    version="0.2.0",
    description="Remove all pests from your project! frenchmaid is a lightweight all platform cli package that will delete all pycache and other folders (contents included) in your project directory. Are you tired of doing it manually each time? Fear not, the frenchmaid will do it for you!",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/lewisjr/frenchmaid.git",
    project_urls={
        "Bug Tracker": "https://github.com/lewisjr/frenchmaid/issues",
    },
    author = "Techtronics Solutions Limited | Lewis Mosho Jr",
    author_email = "lmosho@techtronicsltd.com",
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Topic :: Utilities",
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Typing :: Typed"
    ],
    packages=["frenchmaid", "frenchmaid.modules", "frenchmaid.switches"],
    include_package_data=True,
    install_requires=["click", "colorama", "typer"],
    entry_points={
        "console_scripts": [
            "frenchmaid=frenchmaid.__main__:main",
        ]
    },
    package_dir={"": "src"},
    python_requires=">=3.0",
)


import os, shutil, sys

platf: str = sys.platform

def delDir(dir_select: str) -> bool:
    try:
        shutil.rmtree(dir_select)
        return True
    except:
        return False

def loopDirsOne(main_dir: str, dir_arr: list[str], num_pycache: int, dir_arr_used: list[str], dir_to_delete: str) -> list[str]:
    iterats: int = num_pycache
    arr: list[str] = dir_arr_used
    arr2: list[str] = dir_arr_used

    if "pyvenv.cfg" not in dir_arr:
        for dir in dir_arr:
            if (dir in dir_arr_used) == False:
                if str(dir) == dir_to_delete:

                    if platf == "win32":
                        arr.append(f"{main_dir}\\{dir}")
                    else:
                        arr2.append(f"{main_dir}/{dir}")

                    iterats += 1
                elif str(dir) != dir_to_delete:

                    other_dir: str = ''

                    if platf == "win32":
                        other_dir = f"{main_dir}\\{dir}"
                    else:
                        other_dir = f"{main_dir}/{dir}"

                    try:
                        other_dirs: list[str] = os.listdir(other_dir)

                        if platf == "win32":
                            iterats_two = loopDirsOne(other_dir, other_dirs, iterats, arr, dir_to_delete)
                        else:
                            iterats_two = loopDirsOne(other_dir, other_dirs, iterats, arr2, dir_to_delete)

                        iterats = iterats + len(iterats_two)
                    except:
                        iterats = iterats


    if platf == "win32":
        return arr
    else:
        return arr2

def loopDirsAll(main_dir: str, dir_arr: list[str], num_pycache: int, dir_arr_used: list[str]) -> list[str]:
    iterats: int = num_pycache
    arr: list[str] = dir_arr_used
    arr2: list[str] = dir_arr_used

    if "pyvenv.cfg" not in dir_arr:
        for dir in dir_arr:
            if (dir in dir_arr_used) == False:
                if str(dir) == "__pycache__" or str(dir) == '.mypy_cache':

                    if platf == "win32":
                        arr.append(f"{main_dir}\\{dir}")
                    else:
                        arr2.append(f"{main_dir}/{dir}")

                    iterats += 1
                elif str(dir) != "__pycache__" or str(dir) != '.mypy_cache':

                    other_dir: str = ''

                    if platf == "win32":
                        other_dir = f"{main_dir}\\{dir}"
                    else:
                        other_dir = f"{main_dir}/{dir}"

                    try:
                        other_dirs: list[str] = os.listdir(other_dir)

                        if platf == "win32":
                            iterats_two = loopDirsAll(other_dir, other_dirs, iterats, arr)
                        else:
                            iterats_two = loopDirsAll(other_dir, other_dirs, iterats, arr2)

                        iterats = iterats + len(iterats_two)
                    except:
                        iterats = iterats

    if platf == "win32":
        return arr
    else:
        return arr2

def clean(dir_main: str, folder: str) -> None:
    try:
        dir_name: str = ''
        
        if platf == "win32":
            dir_name = dir_main[str(dir_main).rfind("\\")+1:len(dir_main)]
        else:
            dir_name = dir_main[str(dir_main).rfind("/")+1:len(dir_main)]
            
        dirs: list[str] = os.listdir(dir_main)
        num_dirs: int = len(dirs)
        arr: list[str] = []
        array_thing: list[str] = []

        if folder == 'all':
            array_thing = loopDirsAll(dir_main, dirs, 0, arr)
        else:
           array_thing = loopDirsOne(dir_main, dirs, 0, arr, folder)

        res: int = len(array_thing)
        
        for official_dir in array_thing:
            delDir(official_dir)

        if res == 0:
            print(f'''\nThere are no pests in your "{dir_name}" directory!''')
        else:
            print(f'\nSuccesfully found {res} pest(s) in {num_dirs} directories and/or files contained within your "{dir_name}" directory. They have been eradicated!\n')

        return

    except Exception as e:
        print('\nflail')
        print(e)
        print('\n')
        return



__app_name__ = "frenchmaid"
__version__ = "0.2.0"


(
    SUCCESS,
    DIR_ERROR,
    DEL_ERROR,
) = range(3)

ERRORS = {
    DIR_ERROR: "Failed to find directory",
    DEL_ERROR: "Failed to delete directory"
}